@@ -21,10 +21,11 @@ import (
 	"log"
 	"os"
 	"path/filepath"
+	"strconv"
 	"strings"
 
 	"github.com/hashicorp/hcl"
-	svchost "github.com/hashicorp/terraform-svchost"
+	"github.com/opentofu/svchost"
 
 	"github.com/opentofu/opentofu/internal/tfdiags"
 )
@@ -37,9 +38,6 @@ const pluginCacheMayBreakLockFileEnvVar = "TF_PLUGIN_CACHE_MAY_BREAK_DEPENDENCY_
 // This is not the configuration for OpenTofu itself. That is in the
 // "config" package.
 type Config struct {
-	Providers    map[string]string
-	Provisioners map[string]string
-
 	// If set, enables local caching of plugins in this directory to
 	// avoid repeatedly re-downloading over the Internet.
 	PluginCacheDir string `hcl:"plugin_cache_dir"`
@@ -60,6 +58,15 @@ type Config struct {
 	Credentials        map[string]map[string]interface{}   `hcl:"credentials"`
 	CredentialsHelpers map[string]*ConfigCredentialsHelper `hcl:"credentials_helper"`
 
+	// RegistryProtocols contains some settings for tailoring the request
+	// timeout and retry count for metadata requests made by our registry
+	// protocol clients.
+	//
+	// [LoadConfig] guarantees that this will always be present and contain
+	// values. If these settings are not configured either in files or the
+	// environment then we use some reasonable default settings.
+	RegistryProtocols *RegistryProtocolsConfig
+
 	// ProviderInstallation represents any provider_installation blocks
 	// in the configuration. Only one of these is allowed across the whole
 	// configuration, but we decode into a slice here so that we can handle
@@ -125,7 +132,14 @@ func LoadConfig(_ context.Context) (*Config, tfdiags.Diagnostics) {
 		if _, err := os.Stat(mainFilename); err == nil {
 			mainConfig, mainDiags := loadConfigFile(mainFilename)
 			diags = diags.Append(mainDiags)
-			config = config.Merge(mainConfig)
+			// NOTE: The order of arguments to merge below seems confusing
+			// given that below we tend to do it the other way around, but
+			// this was a historical inconsistency that we only discovered
+			// after making BuiltinConfig not just be completely empty and
+			// so we've swapped these but left the others unswapped to avoid
+			// changing those behaviors. It was unfortunately never well
+			// specified what is overriding what here.
+			config = mainConfig.Merge(config)
 		}
 	} else {
 		diags = diags.Append(mainFileDiags)
@@ -191,6 +205,9 @@ func loadConfigFile(path string) (*Config, tfdiags.Diagnostics) {
 	// using a structure that is not compatible with HCL 1's DecodeObject,
 	// or HCL 1 would be too liberal in parsing and thus make it harder
 	// for us to potentially transition to using HCL 2 later.
+	registryProtocolsConfig, registryProtocolsDiags := decodeRegistryProtocolsConfigFromConfig(obj)
+	diags = diags.Append(registryProtocolsDiags)
+	result.RegistryProtocols = registryProtocolsConfig
 	providerInstBlocks, providerInstDiags := decodeProviderInstallationFromConfig(obj)
 	diags = diags.Append(providerInstDiags)
 	result.ProviderInstallation = providerInstBlocks
@@ -201,14 +218,6 @@ func loadConfigFile(path string) (*Config, tfdiags.Diagnostics) {
 	diags = diags.Append(ociCredsDiags)
 	result.OCIRepositoryCredentials = ociCredsBlocks
 
-	// Replace all env vars
-	for k, v := range result.Providers {
-		result.Providers[k] = os.ExpandEnv(v)
-	}
-	for k, v := range result.Provisioners {
-		result.Provisioners[k] = os.ExpandEnv(v)
-	}
-
 	if result.PluginCacheDir != "" {
 		result.PluginCacheDir = os.ExpandEnv(result.PluginCacheDir)
 	}
@@ -274,6 +283,11 @@ func envConfig(env map[string]string) *Config {
 		config.PluginCacheMayBreakDependencyLockFile = true
 	}
 
+	// The environment config _always_ has opinions about the registry
+	// protocols, because we include the default values in here if the
+	// relevant environment variables aren't set.
+	config.RegistryProtocols = decodeRegistryProtocolsConfigFromEnvironment()
+
 	return config
 }
 
@@ -381,26 +395,6 @@ func (c *Config) Validate() tfdiags.Diagnostics {
 // new configuration with the two merged.
 func (c *Config) Merge(c2 *Config) *Config {
 	var result Config
-	result.Providers = make(map[string]string)
-	result.Provisioners = make(map[string]string)
-	for k, v := range c.Providers {
-		result.Providers[k] = v
-	}
-	for k, v := range c2.Providers {
-		if v1, ok := c.Providers[k]; ok {
-			log.Printf("[INFO] Local %s provider configuration '%s' overrides '%s'", k, v, v1)
-		}
-		result.Providers[k] = v
-	}
-	for k, v := range c.Provisioners {
-		result.Provisioners[k] = v
-	}
-	for k, v := range c2.Provisioners {
-		if v1, ok := c.Provisioners[k]; ok {
-			log.Printf("[INFO] Local %s provisioner configuration '%s' overrides '%s'", k, v, v1)
-		}
-		result.Provisioners[k] = v
-	}
 
 	result.PluginCacheDir = c.PluginCacheDir
 	if result.PluginCacheDir == "" {
@@ -446,6 +440,8 @@ func (c *Config) Merge(c2 *Config) *Config {
 		}
 	}
 
+	result.RegistryProtocols = mergeRegistryProtocolConfigs(c2.RegistryProtocols, c.RegistryProtocols)
+
 	if (len(c.ProviderInstallation) + len(c2.ProviderInstallation)) > 0 {
 		result.ProviderInstallation = append(result.ProviderInstallation, c.ProviderInstallation...)
 		result.ProviderInstallation = append(result.ProviderInstallation, c2.ProviderInstallation...)
@@ -506,3 +502,23 @@ func cliConfigFileOverride() string {
 	}
 	return configFilePath
 }
+
+const (
+	// providerDownloadRetryCountEnvName is the environment variable name used to customize
+	// the HTTP retry count for module downloads.
+	providerDownloadRetryCountEnvName = "TF_PROVIDER_DOWNLOAD_RETRY"
+
+	providerDownloadDefaultRetry = 2
+)
+
+// ProviderDownloadRetries will attempt for requests with retryable errors, like 502 status codes
+func ProviderDownloadRetries() int {
+	r := providerDownloadDefaultRetry
+	if v := os.Getenv(providerDownloadRetryCountEnvName); v != "" {
+		retry, err := strconv.Atoi(v)
+		if err == nil && retry > 0 {
+			r = retry
+		}
+	}
+	return r
+}
