@@ -8,7 +8,9 @@ package tofu
 import (
 	"context"
 	"fmt"
+	"log"
 	"strings"
+	"sync"
 
 	"github.com/opentofu/opentofu/internal/addrs"
 	"github.com/opentofu/opentofu/internal/dag"
@@ -48,25 +50,32 @@ type nodeExpandPlannableResource struct {
 	// structure in the future, as we need to compare for equality and take the
 	// union of multiple groups of dependencies.
 	dependencies []addrs.ConfigResource
+
+	// This slice is meant to keep references to the resourceCloser's of the expanded instances.
+	// Later, this will be called from nodeCloseableResource.
+	// At the time of introducing this, it was strictly meant for ephemeral resources, but if there
+	// will be other closeable resources, this could be used for those too.
+	closers []resourceCloser
 }
 
 var (
-	_ GraphNodeDestroyerCBD         = (*nodeExpandPlannableResource)(nil)
-	_ GraphNodeDynamicExpandable    = (*nodeExpandPlannableResource)(nil)
-	_ GraphNodeReferenceable        = (*nodeExpandPlannableResource)(nil)
-	_ GraphNodeReferencer           = (*nodeExpandPlannableResource)(nil)
-	_ GraphNodeConfigResource       = (*nodeExpandPlannableResource)(nil)
-	_ GraphNodeAttachResourceConfig = (*nodeExpandPlannableResource)(nil)
-	_ GraphNodeAttachDependencies   = (*nodeExpandPlannableResource)(nil)
-	_ GraphNodeTargetable           = (*nodeExpandPlannableResource)(nil)
-	_ graphNodeExpandsInstances     = (*nodeExpandPlannableResource)(nil)
+	_ GraphNodeDestroyerCBD                          = (*nodeExpandPlannableResource)(nil)
+	_ GraphNodeDynamicExpandable                     = (*nodeExpandPlannableResource)(nil)
+	_ GraphNodeReferenceable                         = (*nodeExpandPlannableResource)(nil)
+	_ GraphNodeReferencer                            = (*nodeExpandPlannableResource)(nil)
+	_ GraphNodeConfigResource                        = (*nodeExpandPlannableResource)(nil)
+	_ GraphNodeAttachResourceConfig                  = (*nodeExpandPlannableResource)(nil)
+	_ GraphNodeAttachDependencies                    = (*nodeExpandPlannableResource)(nil)
+	_ GraphNodeTargetable                            = (*nodeExpandPlannableResource)(nil)
+	_ graphNodeRetainedByPruneUnusedNodesTransformer = (*nodeExpandPlannableResource)(nil)
+	_ closableResource                               = (*nodeExpandPlannableResource)(nil)
 )
 
 func (n *nodeExpandPlannableResource) Name() string {
 	return n.NodeAbstractResource.Name() + " (expand)"
 }
 
-func (n *nodeExpandPlannableResource) expandsInstances() {
+func (n *nodeExpandPlannableResource) retainDuringUnusedPruning() {
 }
 
 // GraphNodeAttachDependencies
@@ -156,14 +165,20 @@ func (n *nodeExpandPlannableResource) DynamicExpand(evalCtx EvalContext) (*Graph
 	// We do it here before expanding the resources in the modules, to avoid running this resolution multiple times
 	importResolver := evalCtx.ImportResolver()
 	var diags tfdiags.Diagnostics
+	// If the import target originated from the import command (instead of the import block), we don't need to
+	// resolve the import as it's already in the resolved form. But it still requires to be validated after the graph walk.
+	// The following loop adds CLI import targets for validation and expands and resolves config import targets.
 	for _, importTarget := range n.importTargets {
-		// If the import target originates from the import command (instead of the import block), we don't need to
-		// resolve the import as it's already in the resolved form
-		// In addition, if PreDestroyRefresh is true, we know we are running as part of a refresh plan, immediately before a destroy
+		// We add CLI import targets to the import resolver. Those targets are validated after the graph walk in Context.Import method.
+		// This was added to correctly validate the existence of the targeted resource instances in case for_each key is used, either on a module or on a resource.
+		if importTarget.IsFromImportCommandLine() {
+			importResolver.addCLIImportTarget(importTarget)
+			continue
+		}
+		// If we have import from import block and PreDestroyRefresh is true, we know we are running as part of a refresh plan, immediately before a destroy
 		// plan. In the destroy plan mode, import blocks are not relevant, that's why we skip resolving imports
-		skipImports := importTarget.IsFromImportBlock() && !n.preDestroyRefresh
-		if skipImports {
-			err := importResolver.ExpandAndResolveImport(importTarget, evalCtx)
+		if importTarget.IsFromImportBlock() && !n.preDestroyRefresh {
+			err := importResolver.ExpandAndResolveImport(context.TODO(), importTarget, evalCtx)
 			diags = diags.Append(err)
 		}
 	}
@@ -221,7 +236,7 @@ func (n *nodeExpandPlannableResource) expandResourceInstances(ctx context.Contex
 	// writeResourceState is responsible for informing the expander of what
 	// repetition mode this resource has, which allows expander.ExpandResource
 	// to work below.
-	moreDiags := n.writeResourceState(moduleCtx, resAddr)
+	moreDiags := n.writeResourceState(ctx, moduleCtx, resAddr)
 	diags = diags.Append(moreDiags)
 	if moreDiags.HasErrors() {
 		return diags.ErrWithWarnings()
@@ -378,6 +393,11 @@ func (n *nodeExpandPlannableResource) resourceInstanceSubgraph(ctx context.Conte
 		if resolvedImportTarget != nil {
 			m.importTarget = *resolvedImportTarget
 		}
+		// When creating concrete instance nodes for the ephemeral resources we want to collect all the
+		// resourceCloser callbacks from the nodes to be able to close the resources at the end of the graph walk.
+		if a.Addr.Resource.Resource.Mode == addrs.EphemeralResourceMode {
+			n.closers = append(n.closers, m.Close)
+		}
 
 		return m
 	}
@@ -435,6 +455,31 @@ func (n *nodeExpandPlannableResource) resourceInstanceSubgraph(ctx context.Conte
 		Steps: steps,
 		Name:  "nodeExpandPlannableResource",
 	}
-	graph, graphDiags := b.Build(context.TODO(), addr.Module)
+	graph, graphDiags := b.Build(ctx, addr.Module)
 	return graph, diags.Append(graphDiags).ErrWithWarnings()
 }
+
+// Close implements closableResource
+func (n *nodeExpandPlannableResource) Close() (diags tfdiags.Diagnostics) {
+	if n.Addr.Resource.Mode != addrs.EphemeralResourceMode {
+		return diags
+	}
+
+	var wg sync.WaitGroup
+	diagsCh := make(chan tfdiags.Diagnostics, len(n.closers))
+	log.Printf("[TRACE] nodeExpandPlannableResource - scheduling %d closing operations for of ephemeral resource %s", len(n.closers), n.Addr.String())
+	// NOTE: since go v1.22 there is no need to copy the loop variable.
+	for _, cb := range n.closers {
+		wg.Add(1)
+		go func() {
+			defer wg.Done()
+			diagsCh <- cb()
+		}()
+	}
+	wg.Wait()
+	close(diagsCh)
+	for d := range diagsCh {
+		diags = diags.Append(d)
+	}
+	return diags
+}
