@@ -342,7 +342,7 @@ func (n *NodePlannableResourceInstance) managedResourceExecute(ctx context.Conte
 	// The import process handles its own refresh
 	if !n.skipRefresh && !importing {
 		s, refreshDiags := n.refresh(ctx, evalCtx, states.NotDeposed, instanceRefreshState)
-		diags = diags.Append(maybeImproveResourceInstanceDiagnostics(refreshDiags, addr))
+		diags = diags.Append(refreshDiags)
 		if diags.HasErrors() {
 			return diags
 		}
@@ -380,7 +380,7 @@ func (n *NodePlannableResourceInstance) managedResourceExecute(ctx context.Conte
 		change, instancePlanState, repeatData, planDiags := n.plan(
 			ctx, evalCtx, nil, instanceRefreshState, n.ForceCreateBeforeDestroy, n.forceReplace,
 		)
-		diags = diags.Append(maybeImproveResourceInstanceDiagnostics(planDiags, addr))
+		diags = diags.Append(planDiags)
 		if diags.HasErrors() {
 			// If we are importing and generating a configuration, we need to
 			// ensure the change is written out so the configuration can be
